/*
 Copyright (C) 2011 Nicholas Peterson. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
 
 * Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
 
 * Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.
 
 * Neither the name of the author nor the names of its contributors may be used
 to endorse or promote products derived from this software without specific
 prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "DemoServer.h"

@implementation DemoServer
static DemoServer *sharedDemoServer = nil;


+ (DemoServer *)sharedServer{	
	if(sharedDemoServer == nil){
		sharedDemoServer = [[DemoServer alloc] init];
	}
    return sharedDemoServer;
}
#pragma mark - Overrides
- (NSString *)baseURL{
    return kServerBaseURL;

}

-(id)parseRecievedData:(NSData *)data{
    /* Call Super to parse json data with Stig Brautasets SBJson library 
        if you want to use your own parsing methods, just dont call super. 
     */
   NSDictionary *parsed = [super parseRecievedData:data];
    
    /* This is where you can validate the parsed data, For instance if the server returns 
     panic info in an [error] key, you can detect that here and return an NSError object.
     
     Returning a Parsed Dictionary will be returned to consumer via 
     - (void)rpConnection:(RPURLConnection *)connection didFinishWithInfo:(NSDictionary *)info;
     
     Returning a NSError will be returned to consumer via 
     - (void)rpConnection:(RPURLConnection *)connection didFailWithError:(NSError *)error;
     
     Returning any other object will be returned via
     - (void)rpConnection:(RPURLConnection *)connection didFinishWithObject:(id)object;
     */
    
    return parsed;
}

- (void)callServerWithTag:(NSString *)tag endpoint:(NSString *)endpoint method:(RPAsyncServerMethod)method 
                    query:(NSDictionary *)query headers:(NSDictionary *)headers consumer:(id)consumer{


    /* This is a good place to globally adjust things 
     this can come in handy for APIs that require an APIKey for every interaction.
     For example:
     
     NSMutableDictionary *adjustedQuery = nil;
     adjustedQuery = [[query mutableCopy] autorelease];
     [adjustedQuery setValue:@"<YOUR_API_KEY>" forKey:@"api"];
     [super callServerWithTag:tag endpoint:endpoint method:method query:adjustedQuery headers:headers consumer:consumer];
     
     
     This is also a good place to globally validate data, 
     of course it's best to perform most user validation in the EntryPoint
     */

    [super callServerWithTag:tag endpoint:endpoint method:method query:query headers:headers consumer:consumer];
}

#pragma mark - EntryPoints
/* All of your EntryPoints (methods that call the server) should call the longest and ugliest method call ever.
 - (void)callServerWithTag:(NSString *)tag 
 endpoint:(NSString *)endpoint 
 method:(RPAsyncServerMethod)method 
 query:(NSDictionary *)query 
 headers:(NSDictionary *)headers consumer:(id)consumer;
 
 Each entrypoint should also take a "consumer" that conforms to 
 <RPAsyncServerDelegate> (or a protocal that conforms to it)
 
 */

-(void)getTweetsFromUser:(NSString *)username consumer:(id<RPAsyncServerDelegate>)consumer{
    if (!username) return;
    [username stringByReplacingOccurrencesOfString:@"@" withString:@""];
    
    username = [NSString stringWithFormat:@"from@%@", username];
    NSDictionary *query = [NSDictionary dictionaryWithObject:username forKey:@"q"];
    [self callServerWithTag:@"getTweets" endpoint:@"search.json" method:RPAsyncServerMethodGet query:query headers:nil consumer:consumer];
}

-(void)getFailWhaleWithConsumer:(id<RPAsyncServerDelegate>)consumer{
   
    [self callServerWithTag:@"getTweets" endpoint:@"search.json" method:RPAsyncServerMethodGet query:nil headers:nil consumer:consumer];
}

@end
