/*
 Copyright (C) 2010, 2011 Nicholas Peterson. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
 
 * Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
 
 * Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.
 
 * Neither the name of the author nor the names of its contributors may be used
 to endorse or promote products derived from this software without specific
 prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "RPAsyncServerBase.h"

@interface RPAsyncServer (private)
- (NSString *) _RPcreateQueryString:(NSDictionary *)params;
@end

@implementation RPAsyncServer
@synthesize delegate;
@synthesize connectionData;
@synthesize historyCount;
@synthesize bytesCount;

static RPAsyncServer *sharedAsyncServer = nil;
-(RPAsyncServer *)init{
	if ((self = [super init])) {
		connectionData = [[NSMutableDictionary alloc]init];		
	}
	return self;
}


+ (RPAsyncServer *)sharedServer{	
	if(sharedAsyncServer == nil){
		sharedAsyncServer = [[RPAsyncServer alloc] init];
	}
    return sharedAsyncServer;
}
#pragma mark - Private Methods
- (NSString *) _RPcreateQueryString:(NSDictionary *)params{
    NSMutableString *post = [[NSMutableString alloc] init];
    // Build query string
    for (NSString* key in params) {
        NSString *value;
        if ([[params valueForKey:key] isKindOfClass:[NSString class]]) {
            value = [params valueForKey:key];
        }else if([[params valueForKey:key] isKindOfClass:[NSArray class]]){           
            for (NSString *aVal in [params valueForKey:key]) {
                if ([post length]>0) {
                    [post appendString:@"&"];
                }
                [post appendFormat:@"%@=%@",
                 [key stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding],
                 [[aVal description] stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding]];
            }
            continue;
        }
        else{
            value = [[params valueForKey:key]description];
        }
        
		if ([post length]>0) {
			[post appendString:@"&"];
		}
		[post appendFormat:@"%@=%@",
         [key stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding],
         [value stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding]];
	}
    
    //clean up
    NSString *immutablePost = [post copy];
    [post release]; post = nil;
    return [immutablePost autorelease];
}
#pragma mark - subclass overrides
// TODO: NOTE IN HEADER
- (NSString *)baseURL{
    NSLog(@"WARNING: subclasses should implement baseURL");
return @"example.com";
}

-(id)parseRecievedData:(NSData *)data{
    if(!data){
        NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
                                  @"No Data",NSLocalizedDescriptionKey,nil];
        
        return [NSError errorWithDomain:RPAsyncServerErrorDomain code:4000 userInfo:userInfo];
    }
    NSString *response = nil;
    NSDictionary *responseDictionary = nil;
    
    @try {
        response = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
        responseDictionary = [response JSONValue]; 
    }
    @catch (NSException *exception) {
        NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
                                  exception.reason,NSLocalizedDescriptionKey,
                                  response, NSLocalizedFailureReasonErrorKey, nil];
        
        return [NSError errorWithDomain:RPAsyncServerErrorDomain code:5000 userInfo:userInfo];
    }
    [response release];
    return responseDictionary;
}


#pragma mark -
#pragma mark Connection Methods
//!!!: Unused.
-(BOOL)connectionInProgress:(NSString *)tag{
	if([connectionData objectForKey:tag]){
		return YES;
	}
	return NO;
}

- (void)callServerWithTag:(NSString *)tag endpoint:(NSString *)endpoint method:(RPAsyncServerMethod)method 
                  query:(NSDictionary *)query headers:(NSDictionary *)headers consumer:(id)consumer{

	NSURLRequest *request = [self createRequestWithEndpoint:(NSString *)endpoint 
                                                     method:(RPAsyncServerMethod)method 
                                                      query:(NSDictionary *)query 
                                                    headers:(NSDictionary *)headers];

#if TARGET_IPHONE_SIMULATOR
	NSLog(@"SERVER: Server Called: %@", request.URL);
#endif

	[self createConnectionWithRequest:request tag:tag consumer:consumer];	
}

-(NSURLRequest *)createRequestWithEndpoint:(NSString *)endpoint method:(RPAsyncServerMethod)method query:(NSDictionary *)query headers:(NSDictionary *)headers{
    NSData *postData = nil;
    
    
	//Set up request
	NSString *urlString = [NSString stringWithFormat:@"%@%@",[self baseURL],endpoint];
    NSURL *url = [NSURL URLWithString:urlString];
    if (!url) return nil; //url String malformed.
    
	NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:url cachePolicy:NSURLRequestReloadIgnoringLocalCacheData timeoutInterval:10];

	//Set up post
	NSString *queryString = [self _RPcreateQueryString:query];
    if ([queryString length]>0) {
        if(method == RPAsyncServerMethodGet){
            NSURL *url = [NSURL URLWithString:[urlString stringByAppendingFormat:@"?%@",queryString]];
            [request setURL:url];
            if (!url) return nil; //url String malformed.
        }else{
        	postData = [queryString dataUsingEncoding:NSUTF8StringEncoding allowLossyConversion:YES];
        }

    }
	NSString *postLength = [NSString stringWithFormat:@"%d", [postData length]];
	self.bytesCount = self.bytesCount+[postData length];
	//set up header info
	switch (method) {
		case RPAsyncServerMethodGet:
			[request setHTTPMethod:@"GET"];
			break;
		case RPAsyncServerMethodPost:
			[request setHTTPMethod:@"POST"];
			break;
		case RPAsyncServerMethodPut:
			[request setHTTPMethod:@"PUT"];
			break;
		case RPAsyncServerMethodDelete:
			[request setHTTPMethod:@"DELETE"];
			break;
		default:
			[request setHTTPMethod:@"GET"];
			break;
	}
    [request setValue:@"application/x-www-form-urlencoded" forHTTPHeaderField:@"Content-Type"];

    if(headers){
        for (NSString* head in headers) {
            if ([[headers valueForKey:head] isKindOfClass:[NSString class]]) 
                [request setValue:[headers valueForKey:head] forHTTPHeaderField:head];
        }
    }

	[request setValue:postLength forHTTPHeaderField:@"Content-Length"];
	[request setTimeoutInterval:15];
	[request setHTTPBody:postData];
    return request;
}

- (NSMutableData *)dataForConnection:(RPURLConnection *)connection{
	NSMutableData *data = [connectionData objectForKey:connection.tag];
	return data;
}


- (void)createConnectionWithRequest:(NSURLRequest *)request tag:(NSString *)tag consumer:(id)consumer{
	RPURLConnection *connection = [[RPURLConnection alloc] initWithRequest:request delegate:self];
	connection.tag = tag;
    connection.URL = request.URL;
	connection.consumer = consumer;
	[connection start]; 
	if (connection) { //analyzer thinks this object does not get released.
#if DEBUG_SERVER
		NSLog(@"connection (%@) started",tag); //silent on device.
#endif
        //Its possable that the mutable data object here may need to be retained 
        NSMutableData *dataHolder = [[NSMutableData alloc] init];
		[connectionData setObject:dataHolder forKey:connection.tag];
        [dataHolder release];
		[UIApplication sharedApplication].networkActivityIndicatorVisible=YES;
	}
	[connection release];
}

- (void)connection:(RPURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
    if ([connection.consumer respondsToSelector:@selector(rpConnectionShouldContinue:)]) {
        if (![connection.consumer rpConnectionShouldContinue:connection]) {
            [connection cancel];
            NSDictionary *errorInfo = [NSDictionary dictionaryWithObject:@"Consumer canceled connection"
                                                                  forKey:NSLocalizedDescriptionKey];
            NSError *statusError = [NSError errorWithDomain:RPAsyncServerErrorDomain
                                                       code:5200
                                                   userInfo:errorInfo];
            [self connection:connection didFailWithError:statusError];
            return;
        }
    }
    
    if ([response respondsToSelector:@selector(statusCode)])
    {
        int statusCode = [((NSHTTPURLResponse *)response) statusCode];
        if(statusCode >= 400){
        NSDictionary *errorInfo = [NSDictionary dictionaryWithObject:[NSString stringWithFormat:
                                              NSLocalizedString(@"Server returned status code %d",@""),
                                              statusCode]
                                      forKey:NSLocalizedDescriptionKey];
        NSError *statusError = [NSError errorWithDomain:RPAsyncServerErrorDomain
                              code:statusCode+2000
                          userInfo:errorInfo];
            [self sendError:statusError forConnection:connection];
        }
    }
    if ([connection.consumer respondsToSelector:@selector(rpConnection:didReceiveResponse:)]) {
        [connection.consumer rpConnection:connection didReceiveResponse:response];
    }
    if ([response isKindOfClass:[NSHTTPURLResponse class]]) 
        connection.response = (NSHTTPURLResponse *)response;
    
    NSMutableData *data = [self dataForConnection:connection];
    [data setLength:0];
}

-(void)sendError:(NSError *)error forConnection:(RPURLConnection *)connection{
    id consumer = connection.consumer;
    if (![consumer respondsToSelector:@selector(rpConnection:didFailWithError:)]) return;
    NSDate *d = [NSDate date];    
    NSDictionary *errorInfo = [NSDictionary dictionaryWithObjectsAndKeys:
                               [error localizedDescription], NSLocalizedDescriptionKey,
                               connection.tag, RPAsyncServerKeyTag,
                               connection.URL, RPAsyncServerKeyURL,
                               d, RPAsyncServerKeyDate,nil];
    
    NSError *newError = [NSError errorWithDomain:[error domain] code:[error code] userInfo:errorInfo];
    
    [consumer rpConnection:connection didFailWithError:newError];
}

- (void)connection:(RPURLConnection *)connection didFailWithError:(NSError *)error{
	[self sendError:error forConnection:connection];
	[connection cancel];
	[connectionData removeObjectForKey:connection.tag];
	
	
	if ([connectionData count]==0) {
		[UIApplication sharedApplication].networkActivityIndicatorVisible=NO;
	}	

}
- (void)connectionDidFinishLoading:(RPURLConnection *)connection
{
	id consumer = connection.consumer;
	NSMutableData *currentData = [self dataForConnection:connection];
	self.bytesCount = self.bytesCount+[currentData length];
	
    id responseDictionary = [self parseRecievedData:currentData];

	if([responseDictionary isKindOfClass:[NSError class]]){
		[self sendError:responseDictionary forConnection:connection];
    }
    if([responseDictionary isKindOfClass:[NSDictionary class]]){
        if ([consumer respondsToSelector:@selector(rpConnection:didFinishWithInfo:)]) {
            [consumer rpConnection:connection didFinishWithInfo:responseDictionary];
        }
    }else{
        if ([consumer respondsToSelector:@selector(rpConnection:didFinishWithObject:)]) {
            [consumer rpConnection:connection didFinishWithObject:responseDictionary];
        }   
    }
    [connectionData removeObjectForKey:connection.tag];
    if ([connectionData count]==0) {
		[UIApplication sharedApplication].networkActivityIndicatorVisible=NO;
	}	
}
- (void)connection:(RPURLConnection *)connection didReceiveData:(NSData *)data
{
	NSMutableData *currentData = [self dataForConnection:connection];	
	[currentData appendData:data];
}

- (void)connection:(NSURLConnection *)connection didReceiveAuthenticationChallenge:(NSURLAuthenticationChallenge *)challenge{
    
    NSLog(@"ERROR: Recieved an unhandled auth challenge");
	[challenge.sender cancelAuthenticationChallenge:challenge]; // Will result in error.
	
}

-(void)dealloc{
	[super dealloc];
	[connectionData release];
}

@end



















